//
// Module: Multiply Add (MADD) Element
//
// Description: Verilog instantiation template for 
// DSP48 embedded MAC blocks arranged as a Systolic
// 23 tap FIR filter. 23 slices used, 1 slice per tap.
// 
//
// Device: Virtex-4 Family
//
// Copyright (c) 2000 Xilinx, Inc.  All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////////
//
			 //	 parameter K = 4;
  // parameter N = 16;

module ACC (PCIN, CLK, LOAD, DOUT);
							 
input             CLK, LOAD;
input   [47:0]    PCIN;

output  [47:0]   	DOUT;

wire    [47:0]   	DOUT_ACC;
wire    [17:0]   	DIN_DELAY_INTERNAL;
wire    [6:0]		OPMODE; 

			  
//          
// XtremeDSP Slice 
//

DSP48 MADD0
  (
    .A(18'b0),             // Input A to Multiplier
    .B(18'b0),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(18'b0),          //
    .PCIN(PCIN),          //
    .OPMODE(OPMODE),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(),         //
    .P(DOUT_ACC),                  //
    .PCOUT()        //
  );

//synthesis attribute AREG of MADD0 is "1";
//synthesis attribute BREG of MADD0 is "1";
//synthesis attribute CARRYINREG of MADD0 is "1";
//synthesis attribute MREG of MADD0 is "1";
//synthesis attribute PREG of MADD0 is "1";
//synthesis attribute OPMODEREG of MADD0 is "1";
//synthesis attribute SUBTRACTREG of MADD0 is "1";
//synthesis attribute CARRYINSELREG of MADD0 is "1";
//synthesis attribute B_INPUT of MADD0 is "DIRECT";

//synthesis translate_off 
defparam MADD0.AREG = 2'b01;
defparam MADD0.BREG = 2'b01;
defparam MADD0.CARRYINREG = 1'b1;
defparam MADD0.MREG = 1'b1;
defparam MADD0.PREG = 1'b1;
defparam MADD0.OPMODEREG = 1'b1;
defparam MADD0.SUBTRACTREG = 1'b1;
defparam MADD0.CARRYINSELREG = 1'b1;
defparam MADD0.B_INPUT = "DIRECT";
//synthesis translate_on

///////////////////////////////////////////////////////////////////
//
//   DELAYS FOR CONTROL PATH (this signal is delayed by 4 cycles to 
//   match the latency through the DSP48 + the required delay. Dont 
//   forget the OPMODE pipeline stage as well.)
//
//////////////////////////////////////////////////////////////////

	reg  SHIFT_REG0, SHIFT_REG1, SHIFT_REG2;
	reg  CAPTURE;
	wire LOAD_INTERNAL;

   always @(posedge CLK)
	begin
			SHIFT_REG2 <= SHIFT_REG1;
			SHIFT_REG1 <= SHIFT_REG0;
         SHIFT_REG0 <= LOAD;
   end

   assign LOAD_INTERNAL = SHIFT_REG2;
   
   assign OPMODE[5] = ~(LOAD_INTERNAL);
   assign OPMODE[4:0] = 5'b00101;
   assign OPMODE[6] = 1'b0;

	always @(posedge CLK)
	begin
           CAPTURE <= LOAD_INTERNAL;
	end

///////////////////////////////////////////////////////////////////
//
//	  CAPTURE REGISTER
//
///////////////////////////////////////////////////////////////////

	reg [40:0] DOUT_WIRE; 

	always @(posedge CLK)
	begin
       if(CAPTURE)
		     DOUT_WIRE <= DOUT_ACC[40:0];
	end

	assign DOUT = DOUT_WIRE;


	endmodule