//------------------------------------------------------------------------------ 
// Copyright (c) 2004 Xilinx, Inc. 
// All Rights Reserved 
//------------------------------------------------------------------------------ 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /   Vendor: Xilinx 
// \   \   \/    Author: Niall Battson, Xilinx, Inc.
//  \   \        Filename: design_top.vhd
//  /   /        Date Last Modified:  Sept 2, 2004 
// /___/   /\    Date Created: Sept 2, 2004 
// \   \  /  \ 
//  \___\/\___\ 
// 
//
// Revision History: 
//------------------------------------------------------------------------------ 
//   
//   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" 
//   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND 
//   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, 
//   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, 
//   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION 
//   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT, 
//   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE 
//   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY 
//  WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE 
//   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR 
//   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF 
//   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE. 
//
//------------------------------------------------------------------------------ 

//
// Module: SEMI-PARALLEL SYSTOLIC 4 MULTIPLIER DISTRIBUTED MEMORY FIR FILTER (16 TAP)
//
// Description: Verilog instantiation template for 
// DSP48 embedded MADD blocks arranged as a Systolic
// 16 tap FIR filter.
// 
//
// Device: Virtex-4
//
// Copyright (c) 2000 Xilinx, Inc.  All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////////
//



module design_top(DIN,CLK,CE,RST,DOUT);
    input [17:0] DIN;
    input CLK;
    input CE;
    input RST;
    output [17:0] DOUT;

	 wire          WE, WE_DELAY_1, WE_DELAY_2, WE_DELAY_3, WE_DELAY_4;
	 wire [1:0]    ADDR, ADDR_DELAY_1, ADDR_DELAY_2, ADDR_DELAY_3, ADDR_DELAY_4;
	 wire [17:0]   DIN_DELAY_1, DIN_DELAY_2, DIN_DELAY_3;
	 wire [47:0]   PCIN_INTERNAL_1, PCIN_INTERNAL_2, PCIN_INTERNAL_3, PCIN_INTERNAL_4;
    wire [47:0]   DOUT_FULL;
	 
	 CONTROL C1
	 (
	     .CLK(CLK),
		  .CE(CE),
		  .RST(RST),
		  .ADDR(ADDR),
		  .WE(WE)
	 );


	 MADD_FIRST M1
	 (
	     .DIN(DIN),
		  .ADDR(ADDR),
		  .CLK(CLK),
		  .WE(WE),
		  .CE(CE),
		  .RST(RST),
		  .DIN_DELAY(DIN_DELAY_1),
		  .DOUT(PCIN_INTERNAL_1),
		  .ADDR_DELAY(ADDR_DELAY_1),
		  .WE_DELAY(WE_DELAY_1)
	 );

	 MADD_ELEMENT2 M2
	 (
	     .DIN(DIN_DELAY_1),
		  .ADDR(ADDR_DELAY_1),
		  .PCIN(PCIN_INTERNAL_1),
		  .CLK(CLK),
		  .WE(WE_DELAY_1),
		  .CE(CE),
		  .RST(RST),
		  .DIN_DELAY(DIN_DELAY_2),
		  .DOUT(PCIN_INTERNAL_2),
		  .ADDR_DELAY(ADDR_DELAY_2),
		  .WE_DELAY(WE_DELAY_2)
	 );

	 MADD_ELEMENT3 M3
	 (
	     .DIN(DIN_DELAY_2),
		  .ADDR(ADDR_DELAY_2),
		  .PCIN(PCIN_INTERNAL_2),
		  .CLK(CLK),
		  .WE(WE_DELAY_2),
		  .CE(CE),
		  .RST(RST),
		  .DIN_DELAY(DIN_DELAY_3),
		  .DOUT(PCIN_INTERNAL_3),
		  .ADDR_DELAY(ADDR_DELAY_3),
		  .WE_DELAY(WE_DELAY_3)
	 );

	 MADD_ELEMENT4 M4
	 (
	     .DIN(DIN_DELAY_3),
		  .ADDR(ADDR_DELAY_3),
		  .PCIN(PCIN_INTERNAL_3),
		  .CLK(CLK),
		  .WE(WE_DELAY_3),
		  .CE(CE),
		  .RST(RST),
		  .DIN_DELAY(DIN_DELAY_3),
		  .DOUT(PCIN_INTERNAL_4),
		  .ADDR_DELAY(ADDR_DELAY_4),
		  .WE_DELAY(WE_DELAY_4)
	 );

	 ACC A1
	 (
	     .PCIN(PCIN_INTERNAL_4),
		  .CLK(CLK),
		  .LOAD(WE_DELAY_4),
		  .DOUT(DOUT_FULL)
	 );

	 assign DOUT = DOUT_FULL[17:0];

endmodule
