//
// Module: Multiply Add (MADD) Element
//
// Description: Verilog instantiation template for 
// DSP48 embedded MAC blocks arranged as a Systolic
// Semi-Parallel 16 tap FIR filter.
// 
//
// Device: Virtex-4 Family
//
// Copyright (c) 2000 Xilinx, Inc.  All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////////
//

module MADD_ELEMENT3 (DIN, ADDR, PCIN, CLK, WE, CE, RST, DIN_DELAY, DOUT, ADDR_DELAY, WE_DELAY);
							 
input             CLK, CE, RST, WE;
input   [1:0]     ADDR;
input   [17:0]    DIN;
input   [47:0]    PCIN;

output			  	WE_DELAY;
output  [1:0]    	ADDR_DELAY;
output  [17:0]   	DIN_DELAY;
output  [47:0]   	DOUT;

wire    [17:0]   	DIN_DELAY_INTERNAL;
wire    [17:0]		COEF; 

reg     [17:0]    DIN_DELAY;


////////////////////////////////////////////////////////////////////////////////////////////
// COEFFICENT VALUES - STORED IN DISTMEM USING CORE													//
//																														//
//-109, -37, 140, 435, 827, 1262, 1663, 1945, 2047, 1945, 1663, 1262, 827, 435, 140, -37	//
////////////////////////////////////////////////////////////////////////////////////////////


											  
//          
// XtremeDSP Slice 
//

DSP48 MADD0
  (
    .A(COEF),             // Input A to Multiplier
    .B(DIN_DELAY),             // Input B to Multiplier
    .C(48'b0),             // Input C to Adder
    .BCIN(18'b0),          //
    .PCIN(PCIN),          //
    .OPMODE(7'b0010101),   //
    .SUBTRACT(1'b0),       //
    .CARRYIN(1'b0),        //
    .CARRYINSEL(2'b00),    //
    .CLK(CLK),             //
    .CEA(CE),            //
    .CEB(CE),            //
    .CEC(CE),            //
    .CEP(CE),            //
    .CEM(CE),            //
    .CECTRL(CE),         //
    .CECARRYIN(CE),      //
	 .CECINSUB(CE),       //
    .RSTA(RST),            //
    .RSTB(RST),            //
    .RSTC(RST),            //
    .RSTP(RST),            //
    .RSTM(RST),            //
    .RSTCTRL(RST),         //
    .RSTCARRYIN(RST),      //
    .BCOUT(),         //
    .P(),                  //
    .PCOUT(DOUT)        //
  );

//synthesis attribute AREG of MADD0 is "1";
//synthesis attribute BREG of MADD0 is "1";
//synthesis attribute CARRYINREG of MADD0 is "1";
//synthesis attribute MREG of MADD0 is "1";
//synthesis attribute PREG of MADD0 is "1";
//synthesis attribute OPMODEREG of MADD0 is "1";
//synthesis attribute SUBTRACTREG of MADD0 is "1";
//synthesis attribute CARRYINSELREG of MADD0 is "1";
//synthesis attribute B_INPUT of MADD0 is "DIRECT";

//synthesis translate_off 
defparam MADD0.AREG = 2'b01;
defparam MADD0.BREG = 2'b01;
defparam MADD0.CARRYINREG = 1'b1;
defparam MADD0.MREG = 1'b1;
defparam MADD0.PREG = 1'b1;
defparam MADD0.OPMODEREG = 1'b1;
defparam MADD0.SUBTRACTREG = 1'b1;
defparam MADD0.CARRYINSELREG = 1'b1;
defparam MADD0.B_INPUT = "DIRECT";
//synthesis translate_on

///////////////////////////////////////////////////////////////////
//
//	  DELAYS FOR CONTROL PATH
//
///////////////////////////////////////////////////////////////////

   reg [1:0] ADDR_DELAY_INTERNAL;
	reg       WE_DELAY_INTERNAL;

   always @(posedge CLK)
	begin
         ADDR_DELAY_INTERNAL <= ADDR;
			WE_DELAY_INTERNAL <= WE;
	end

	assign ADDR_DELAY = ADDR;
	assign WE_DELAY = WE;


///////////////////////////////////////////////////////////////////
//
//   INPUT DELAY BUFFER USING SRL16
//
///////////////////////////////////////////////////////////////////

   reg [17:0] SHIFT_REG1, SHIFT_REG2, SHIFT_REG3, SHIFT_REG4;

   always @(posedge CLK)
      if (WE) 
		   begin
         SHIFT_REG4 <= SHIFT_REG3;
			SHIFT_REG3 <= SHIFT_REG2;
			SHIFT_REG2 <= SHIFT_REG1;
			SHIFT_REG1 <= DIN;
	      end

   assign DOUT_DELAY_INTERNAL = ADDR==2'd0 ? SHIFT_REG1 : 
										 (ADDR==2'd1 ? SHIFT_REG2 :
										 (ADDR==2'd2 ? SHIFT_REG3 :
										 (ADDR==2'd3 ? SHIFT_REG4 : 18'b0)));
   
	always @(posedge CLK)
      if (CE) 
        DIN_DELAY <= DIN_DELAY_INTERNAL;



///////////////////////////////////////////////////////////////////
//
//    COEFFICIENT MEMORY BUFFER USING DISTRIBUTED MEMORY
//
////////////////////////////////////////////////////////////////////
   
 distmem3 mem3 (
    .A(ADDR),
    .CLK(CLK),
    .QSPO_CE(CE),
    .QSPO(COEF));

////////////////////////////////////////////////////////////////////////////////////

endmodule
