library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;

entity acc is

	 Port ( pcin : in std_logic_vector(47 downto 0);
           clk : in std_logic;
           load : in std_logic;
           dout : out std_logic_vector(47 downto 0)
			  );

end acc;

architecture Behavioral of acc is


------------------------------------------------------------------------
--
-- Component Declarations:
--
------------------------------------------------------------------------

component DSP48
  generic
  (
  AREG            : integer       := 1;
  B_INPUT         : string        := "DIRECT";
  BREG            : integer       := 1;
  CARRYINREG      : integer       := 1;
  CARRYINSELREG   : integer       := 1;
  CREG            : integer       := 1;
  LEGACY_MODE     : string        := "MULT18X18S";
  MREG            : integer       := 1;
  OPMODEREG       : integer       := 1;
  PREG            : integer       := 1;
  SUBTRACTREG     : integer       := 1;
  SIM_X_INPUT	   : string	       := "GENERATE_X_ONLY"
  );

      PORT (
         BCOUT                   : OUT std_logic_vector(17 DOWNTO 0);
         P                       : OUT std_logic_vector(47 DOWNTO 0);
         PCOUT                   : OUT std_logic_vector(47 DOWNTO 0);
         A                : in  std_logic_vector(17 downto 0); -- input A to Multiplier 
         B                : in  std_logic_vector(17 downto 0); -- input B to Multiplier 
         C                : in  std_logic_vector(47 downto 0); -- input C to Adder 
         OPMODE                  : IN  std_logic_vector(6 DOWNTO 0);
         SUBTRACT                : IN  std_logic;
         CARRYIN                 : IN  std_logic;
         CARRYINSEL              : IN  std_logic_vector(1 DOWNTO 0);
         BCIN                    : IN  std_logic_vector(17 DOWNTO 0);
         PCIN                    : IN  std_logic_vector(47 DOWNTO 0);
         CLK                     : IN  std_logic;
         CEA                     : IN  std_logic;
         CEB                     : IN  std_logic;
         CEC                     : IN  std_logic;
         CEP                     : IN  std_logic;
         CEM                     : IN  std_logic;
         CECTRL                  : IN  std_logic;
         CECARRYIN               : IN  std_logic;
         CECINSUB						: IN  std_logic;
			RSTA                    : IN  std_logic;
         RSTB                    : IN  std_logic;
         RSTC                    : IN  std_logic;
         RSTP                    : IN  std_logic;
         RSTM                    : IN  std_logic;
         RSTCTRL                 : IN  std_logic;
         RSTCARRYIN              : IN  std_logic);
   END COMPONENT;

-----------------------------------------------------------------------------------
--
--	  SIGNAL DECLARATIONS
--
-----------------------------------------------------------------------------------


   signal dout_acc: std_logic_vector(47 downto 0);
	signal opmode: std_logic_vector(6 downto 0) := "0010010";
	signal ld_d1: std_logic;
	signal ld_d2: std_logic;
	signal ld_d3: std_logic;
	signal ld_d4: std_logic;
	signal load_delayed: std_logic;


begin

   MADD1 : DSP48 
      PORT MAP (
         A => "000000000000000000",
         B => "000000000000000000",
         C => "000000000000000000000000000000000000000000000000",
         BCIN => "000000000000000000",
         PCIN => pcin,
         OPMODE => opmode,
         SUBTRACT => '0',
         CARRYIN => '0',
         CARRYINSEL => "00",
         CLK => CLK,
         CEA => '1',
         CEB => '1',
         CEC => '1',
         CEP => '1',
         CEM => '1',
         CECTRL => '1',
         CECARRYIN => '1',
         CECINSUB => '1',
			RSTA => '0',
         RSTB => '0',
         RSTC => '0',
         RSTP => '0',
         RSTM => '0',
         RSTCTRL => '0',
         RSTCARRYIN => '0',
         BCOUT => open,
         P => dout_acc,
         PCOUT => open);

----------------------------------------------------------------------------------------------------------------------------
--
--	  DELAYS FOR CONTROL PATH (this signal is delayed by 4 cycles to match the latency through the DSP48 + the required delay. Dont forget the OPMODE pipeline stage as well.)
--
----------------------------------------------------------------------------------------------------------------------------

delay_elements : process(clk)
begin
		if (clk'event and clk='1') then 
			 ld_d3 <= ld_d2;
			 ld_d2 <= ld_d1;
			 ld_d1 <= load;
		end if;
end process delay_elements;

output_register: process(clk)
begin
		if (clk'event and clk='1') then 
		 	 load_delayed <= ld_d3;
		end if;
end process output_register;

opmode(1) <= not(ld_d3);

----------------------------------------------------------------------------------------------------------------------------
--
--	  CAPTURE REGISTER
--
----------------------------------------------------------------------------------------------------------------------------

capture_register: process(clk, load_delayed)
begin
		if (clk'event and clk='1') then 
			 if (load_delayed='1') then
		 	    dout <= dout_acc;
          
			 end if;
		end if;
end process capture_register;

end Behavioral;

