-------------------------------------------------------------------------------- 
-- Copyright (c) 2004 Xilinx, Inc. 
-- All Rights Reserved 
-------------------------------------------------------------------------------- 
--   ____  ____ 
--  /   /\/   / 
-- /___/  \  /   Vendor: Xilinx 
-- \   \   \/    Author: Niall Battson, Xilinx, Inc.
--  \   \        Filename: design_top.vhd
--  /   /        Date Last Modified:  Sept 2, 2004 
-- /___/   /\    Date Created: Sept 2, 2004 
-- \   \  /  \ 
--  \___\/\___\ 
-- 
--
-- Revision History: 
-------------------------------------------------------------------------------- 
--   
--   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" 
--   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND 
--   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, 
--   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, 
--   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION 
--   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT, 
--   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE 
--   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY 
--   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE 
--   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR 
--   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF 
--   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
--   FOR A PARTICULAR PURPOSE. 
--
-------------------------------------------------------------------------------- 

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;
library WORK;
use work.filter_param.all;

entity design_top is
    Port ( xn : in std_logic_vector(17 downto 0);
           clk : in std_logic;
           ce : in std_logic;
			  rst : in std_logic;
			  yn : out std_logic_vector(47 downto 0));
end design_top;

architecture Behavioral of design_top is


------------------------------------------------------------------------
--
-- Component Declarations:
--
------------------------------------------------------------------------

component MADD_ELEMENT
    generic (
           opmode: std_logic_vector(6 downto 0);
			  coefficients: coef_per_mult_array);

	 Port ( din : in std_logic_vector(17 downto 0);
           addr : in integer range 0 to 3;
			  pcin : in std_logic_vector(47 downto 0);
			  clk : in std_logic;
           we: in std_logic;
			  ce : in std_logic;
			  rst : in std_logic;
			  din_delay : out std_logic_vector(17 downto 0);
			  dout : out std_logic_vector(47 downto 0);
			  addr_delay : out integer range 0 to 3;
			  we_delay : out std_logic

			  );
  end component;

  component MADD_FIRST
    generic (
           opmode: std_logic_vector(6 downto 0);
			  coefficients: coef_per_mult_array );

	 Port ( din : in std_logic_vector(17 downto 0);
           addr : in integer range 0 to 3;
			  clk : in std_logic;
           we: in std_logic;
			  ce : in std_logic;
			  rst : in std_logic;
			  din_delay : out std_logic_vector(17 downto 0);
			  dout : out std_logic_vector(47 downto 0);
			  addr_delay : out integer range 0 to 3;
			  we_delay : out std_logic
			  );
  end component;

  component ACC
	 Port ( pcin : in std_logic_vector(47 downto 0);
           clk : in std_logic;
           load : in std_logic;
           dout : out std_logic_vector(47 downto 0)
			  );
  end component;

  component CONTROL
	 Port ( clk : in std_logic;
           ce : in std_logic;
			  rst : in std_logic;
           addr : out integer range 0 to (4-1);
           we : out std_logic
			  );

  end component;

---------------------------------------------------------------------------------
--     Setup all the connecting signals between MADD elements in one vector    --
---------------------------------------------------------------------------------
  
  signal pcout_internal_1 : std_logic_vector(47 downto 0);
  signal pcout_internal_2 : std_logic_vector(47 downto 0);
  signal pcout_internal_3 : std_logic_vector(47 downto 0);
  signal pcout_internal_4 : std_logic_vector(47 downto 0);
  signal din_delay_1 : std_logic_vector(17 downto 0);
  signal din_delay_2 : std_logic_vector(17 downto 0);
  signal din_delay_3 : std_logic_vector(17 downto 0);
  signal din_delay_4 : std_logic_vector(17 downto 0);
  signal addr : integer range 0 to 3;
  signal addr_delay_1 : integer range 0 to 3;
  signal addr_delay_2 : integer range 0 to 3;
  signal addr_delay_3 : integer range 0 to 3;
  signal addr_delay_4 : integer range 0 to 3;
  signal we : std_logic;
  signal we_delay_1 : std_logic;
  signal we_delay_2 : std_logic;
  signal we_delay_3 : std_logic;
  signal we_delay_4 : std_logic;

begin
  
  C0: CONTROL
      port map (
		  clk => clk,
        ce => ce,
        rst => rst,
        addr => addr,
        we => we
      );

  
  M1: MADD_FIRST
      generic map (
		  opmode => "0000101",
        coefficients => coef_set1
      )
      port map (
        din => xn,
        addr => addr,
		  clk => clk,
		  we => we,
		  ce => ce,
		  rst => rst,
		  din_delay => din_delay_1, 
		  dout => pcout_internal_1,
		  addr_delay => addr_delay_1,
		  we_delay => we_delay_1
      );
  
  M2: MADD_ELEMENT
      generic map (
        opmode => "0010101",
		  coefficients => coef_set2
      )
      port map (
        din => din_delay_1,
        addr => addr_delay_1,
		  clk => clk,
		  we => we_delay_1,
		  ce => ce,
		  rst => rst,
		  pcin => pcout_internal_1,
		  din_delay => din_delay_2, 
		  dout => pcout_internal_2,
		  addr_delay => addr_delay_2,
		  we_delay => we_delay_2
      );


  M3: MADD_ELEMENT
  	  generic map (
        opmode => "0010101",
		  coefficients => coef_set3
      )
      port map (
        din => din_delay_2,
        addr => addr_delay_2,
		  clk => clk,
		  we => we_delay_2,
		  ce => ce,
		  rst => rst,
		  pcin => pcout_internal_2,
		  din_delay => din_delay_3, 
		  dout => pcout_internal_3,
		  addr_delay => addr_delay_3,
		  we_delay => we_delay_3
      );

   M4: MADD_ELEMENT
  	  generic map (
        opmode => "0010101",
		  coefficients => coef_set4
      )
      port map (
        din => din_delay_3,
        addr => addr_delay_3,
		  clk => clk,
		  we => we_delay_3,
		  ce => ce,
		  rst => rst,
		  pcin => pcout_internal_3,
		  din_delay => din_delay_4, 
		  dout => pcout_internal_4,
		  addr_delay => addr_delay_4,
		  we_delay => we_delay_4
      );

  A1: ACC
      port map (
        pcin => pcout_internal_4,
        clk => clk,
        load => we_delay_4,
        dout => yn
      );
   
end Behavioral;
