library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;
library WORK;
use work.filter_param.all;

entity madd_first is
	 generic (
           opmode: std_logic_vector(6 downto 0) := "0000101";
			  coefficients : coef_per_mult_array);

	 Port ( din : in std_logic_vector(17 downto 0);
           addr : in integer range 0 to 3;
			  clk : in std_logic;
           we: in std_logic;
			  ce : in std_logic;
			  rst : in std_logic;
			  din_delay : out std_logic_vector(17 downto 0);
			  dout : out std_logic_vector(47 downto 0);
			  addr_delay : out integer range 0 to 3;
			  we_delay : out std_logic
			  );
end madd_first;

architecture Behavioral of madd_first is

------------------------------------------------------------------------
--
-- Component Declarations:
--
------------------------------------------------------------------------

component DSP48
-- synthesis translate_off
  generic
  (
  AREG            : integer       := 1;
  B_INPUT         : string        := "DIRECT";
  BREG            : integer       := 1;
  CARRYINREG      : integer       := 1;
  CARRYINSELREG   : integer       := 1;
  CREG            : integer       := 1;
  LEGACY_MODE     : string        := "MULT18X18S";
  MREG            : integer       := 1;
  OPMODEREG       : integer       := 1;
  PREG            : integer       := 1;
  SUBTRACTREG     : integer       := 1;
  SIM_X_INPUT	   : string	       := "GENERATE_X_ONLY"
  );
-- synthesis translate_on

      PORT (
         BCOUT                   : OUT std_logic_vector(17 DOWNTO 0);
         P                       : OUT std_logic_vector(47 DOWNTO 0);
         PCOUT                   : OUT std_logic_vector(47 DOWNTO 0);
         A                : in  std_logic_vector(17 downto 0); -- input A to Multiplier 
         B                : in  std_logic_vector(17 downto 0); -- input B to Multiplier 
         C                : in  std_logic_vector(47 downto 0); -- input C to Adder 
         OPMODE                  : IN  std_logic_vector(6 DOWNTO 0);
         SUBTRACT                : IN  std_logic;
         CARRYIN                 : IN  std_logic;
         CARRYINSEL              : IN  std_logic_vector(1 DOWNTO 0);
         BCIN                    : IN  std_logic_vector(17 DOWNTO 0);
         PCIN                    : IN  std_logic_vector(47 DOWNTO 0);
         CLK                     : IN  std_logic;
         CEA                     : IN  std_logic;
         CEB                     : IN  std_logic;
         CEC                     : IN  std_logic;
         CEP                     : IN  std_logic;
         CEM                     : IN  std_logic;
         CECTRL                  : IN  std_logic;
         CECARRYIN               : IN  std_logic;
         CECINSUB						: IN  std_logic;
			RSTA                    : IN  std_logic;
         RSTB                    : IN  std_logic;
         RSTC                    : IN  std_logic;
         RSTP                    : IN  std_logic;
         RSTM                    : IN  std_logic;
         RSTCTRL                 : IN  std_logic;
         RSTCARRYIN              : IN  std_logic);
   END COMPONENT;

-----------------------------------------------------------------------------------
--
--	  SIGNAL DECLARATIONS
--
-----------------------------------------------------------------------------------


   signal coef: std_logic_vector(17 downto 0);
	signal mem_out: std_logic_vector(17 downto 0);
	signal din_d0: std_logic_vector(17 downto 0);
	signal din_d1: std_logic_vector(17 downto 0);
	signal din_d2: std_logic_vector(17 downto 0);
	signal din_d3: std_logic_vector(17 downto 0);
	signal din_reg: std_logic_vector(17 downto 0);
	signal din_delay_internal: std_logic_vector(17 downto 0);

  type ROM_TABLE is array (0 to 3) of STD_LOGIC_VECTOR (17 downto 0);
  constant ROM : ROM_TABLE := (
        conv_std_logic_vector(coefficients(0), 18),
        conv_std_logic_vector(coefficients(1), 18),
        conv_std_logic_vector(coefficients(2), 18),
        conv_std_logic_vector(coefficients(3), 18)
        ); 


begin

   MADD1 : DSP48 
      PORT MAP (
         A => coef,
         B => din_delay_internal,
         C => "000000000000000000000000000000000000000000000000",
         BCIN => "000000000000000000",
         PCIN => "000000000000000000000000000000000000000000000000",
         OPMODE => opmode,
         SUBTRACT => '0',
         CARRYIN => '0',
         CARRYINSEL => "00",
         CLK => CLK,
         CEA => CE,
         CEB => CE,
         CEC => CE,
         CEP => CE,
         CEM => CE,
         CECTRL => CE,
         CECARRYIN => CE,
         CECINSUB => CE,
			RSTA => RST,
         RSTB => RST,
         RSTC => RST,
         RSTP => RST,
         RSTM => RST,
         RSTCTRL => RST,
         RSTCARRYIN => RST,
         BCOUT => open,
         P => open,
         PCOUT => dout);

-----------------------------------------------------------------------------------
--
--	  DELAYS FOR CONTROL PATH
--
-----------------------------------------------------------------------------------

delay_elements : process(clk)
begin
		if (clk'event and clk='1') then 
		 	 addr_delay <= addr;
			 we_delay <= we;
		end if;
end process delay_elements;

-----------------------------------------------------------------------------------
--
--	  INPUT TIME SERIES DELAY BUFFER
--
-----------------------------------------------------------------------------------

input_buffer : process (clk, we)
begin
		if (clk'event and clk='1') then
			 if (we = '1') then
			    din_d3 <= din_d2;
				 din_d2 <= din_d1;
				 din_d1 <= din_d0;
				 din_d0 <= din;
				 
			 end if;
		end if;
end process input_buffer;

shift_reg_output:
with addr select
      din_reg <= din_d0 when 0,
		           din_d1 when 1,
		           din_d2 when 2,
		           din_d3 when 3;

data_output_register: process(clk)
begin
		if (clk'event and clk='1') then
		    din_delay_internal <= din_reg;
		end if;
end process data_output_register;

din_delay <= din_delay_internal;

-----------------------------------------------------------------------------------
--
--	  COEFFICIENT MEMORY
--
-----------------------------------------------------------------------------------

mem_out <= ROM(addr);

coef_output_register: process(clk)
begin
		if (clk'event and clk='1') then
		        coef <= mem_out;
			
		end if;
end process coef_output_register;


end Behavioral;